def get_params_dict_inlist(data, inList: list) -> dict:
    '''
    整理传入字段，紧包含在inList中的并传入值的字段
    '''
    try:
        adata = data.dict()
    except:
        adata = data
    paramDict = {}
    for i in inList:
        if i in adata.keys():
            if data[i] != "":
                paramDict[i] = adata[i]
            else:
                continue
        else:
            continue
    return paramDict


'''
    return Json models
'''

import functools
from datetime import datetime
from rest_framework.response import Response
from django.db.utils import IntegrityError


class ReCode(object):

    def error_func(self, status, error,message = None):
        
        data = {
            "status": status,  # 状态
            "code": 0,
            "error": error,  # 错误提示
            "message": message if message else error,  # 信息
            "timestamp": datetime.now().strftime('%Y/%m/%d %H:%M:%S'),  # 时间
        }
        return data

    def success_func(self,data: dict):

        re_data = {
            "status": 200,
            "code": 1,
            "message": "操作成功",
            "data": data,
            "timestamp": datetime.now().strftime('%Y/%m/%d %H:%M:%S')
        }
        return re_data


def get_error_handler_return(view_func):
    @functools.wraps(view_func)
    def wrapper(request, *args, **kwargs):
        try:
            return view_func(request, *args, **kwargs)
        except AssertionError as a:
            re_data = ReCode().error_func(status=a.args[0][0], error=a.args[0][1])
            
            return Response(re_data)
        except IntegrityError as t:
            re_data = ReCode().error_func(status=1002, error="内容已存在")
            return Response(re_data)
        except Exception as e:
            re_data = ReCode().error_func(status=5000, error=str(e))
            
            return Response(re_data)
    return wrapper



from rest_framework.response import Response
from rest_framework.views import APIView
from itertools import chain
from datetime import datetime


from utils import recode, get_error_handler_return
from utils.params_to_dict import get_params_dict_inlist
from utils.comfunc import paginationfunc, exam_old, file_save_addr
from utils.params_verify import validator_function

class LminApiView(APIView):

    modelObj: object  # 查询数据库对象
    list_display: tuple or list
    get_list_filter: list # 列表筛选字段  [{"field": , "rule": "field__in"}]
    post_files: dict # post上传文件
    put_files: dict # post上传文件
    unique_param: list # 需要查重的字段
    delete_rule: dict # 删除时传入的字段规则
    post_rule: dict # 新增时传入的字段规则
    put_rule: dict # 编辑时传入的字段规则
    put_where_params = [] # put请求修改时的条件参数
    post_inlist: list # 新增时创建model对象需要的字段
    put_inlist: list # 编辑时修改model对象需要的字段
    file_first_path: str
    delete_where_params = [] # delete请求删除时的条件参数

    def _get_list_values(self, instance):
        opts = instance._meta
        data = {}
        fields = self.list_display
        for f in chain(opts.concrete_fields, opts.private_fields, opts.many_to_many):
            if not getattr(f, 'editable', False):
                continue
            if fields and f.name not in fields:
                continue
            if type(f.value_from_object(instance)) == datetime:
                data[f.name] = datetime.strftime(f.value_from_object(instance), "%Y-%m-%d %H:%M:%S")
            else:
                data[f.name] = f.value_from_object(instance)
        return data

    def _get_list_filter_kwargs(self, data):
        try:
            adata = data.dict()
        except:
            adata = data
        filterDict = {}
        for i in self.get_list_filter:
            key = i["rule"]
            value = adata[i["field"]]
            filterDict[key] = value
        return filterDict

    def _get_post_params(self, data):
        return get_params_dict_inlist(data, self.post_inlist)
    
    def _get_put_params(self, data):
        return get_params_dict_inlist(data, self.put_inlist)

    @get_error_handler_return
    def get(self, request, *args, **kwargs):
        '''
        APIView视图get请求
        param page: 页码
        param size: 每页条数
        return: 返回分两种情况，传入page时进行分页返回data为对象，不传时返回所有结果data为list
        '''
        page = request.GET.get("page", None)
        size = request.GET.get("size", 20)
        se_dict = self._get_list_filter_kwargs(data=request.GET)
        res = self.modelObj.objects.filter(**se_dict)
        resList = []
        for i in res:
            resList.append(self._get_list_values(i))
        if page:
            rList = paginationfunc(resList=resList, pg=page, size=size)
        else:
            rList = resList
        return Response(recode.success_func(data=rList))

    @get_error_handler_return
    def post(self, request, *args, **kwargs):
        '''
        APIView视图post请求
        '''
        ok = validator_function(rules=self.post_rule,request=request)
        if ok != True:
            return ok
        cr_dict = self._get_post_params(data=request.data)
        for unparam in self.unique_param_list:
            val = request.data.get(unparam)
            assert exam_old(obj=self.obj, paramDict={unparam:val}, excludeDict=None),(1001, "该【{}】内容已存在".format(unparam))
        
        for param in self.post_file_params_dict.keys():
            if request.data.get(param):
                file = request.data.get(param)
                ok, fileaddr = file_save_addr(file=file, firstPath=self.file_first_path, pathList=self.post_file_params_dict[param]["path"])
                if ok != True:
                    return Response(recode.error_func(status=5003, error=ok))
                cr_dict[param] = fileaddr
        self.modelObj.objects.create(**cr_dict)
        return Response(recode.success_func(data={}))

    @get_error_handler_return
    def put(self, request, *args, **kwargs):
        '''
        APIView视图put请求
        '''
        assert self.put_where_params, ("update_find where error", "请传入修改条件")
        ok = validator_function(rules=self.put_rule,request=request)
        if ok != True:
            return ok
        up_dict = self._get_put_params(data=request.data)
        for unparam in self.unique_param_list:
            val = request.data.get(unparam)
            assert exam_old(obj=self.obj, paramDict={unparam:val}, excludeDict=None),(1001, "该【{}】内容已存在".format(unparam))
        
        for param in self.post_file_params_dict.keys():
            if request.data.get(param):
                file = request.data.get(param)
                ok, fileaddr = file_save_addr(file=file, firstPath=self.file_first_path, pathList=self.post_file_params_dict[param]["path"])
                if ok != True:
                    return Response(recode.error_func(status=5003, error=ok))
                up_dict[param] = fileaddr
        if up_dict != {}:
            f_dict = get_params_dict_inlist(data=request.data, inList=self.put_where_params)
            ups = self.modelObj.objects.filter(**f_dict)
            for i in ups:
                i.__dict__.update(**up_dict)
                i.save()
        return Response(recode.success_func(data={}))

    @get_error_handler_return
    def delete(self, request, *args, **kwargs):
        '''
        APIView视图delete请求
        '''
        assert self.delete_where_params, ("delete_find where error", "请传入修改条件")
        ok = validator_function(rules=self.delete_rule,request=request)
        if ok != True:
            return ok
        f_dict = get_params_dict_inlist(data=request.data, inList=self.delete_where_params)
        dels = self.modelObj.objects.filter(**f_dict)
        for i in dels:
            i.delete()
        return Response(recode.success_func(data={}))


class TestView(LminApiView):

    modelObj = UsersModels  # 查询数据库对象
    list_display = ("id", "name")
    get_list_filter = [] # 列表筛选字段  [{"field": , "rule": "field__in"}] "rule"根据django orm 的**kwargs查询方式条件来写
    post_files = {} # post上传文件 {"file_field": {"path": ["folder_name", "child_folder_name"]}}
    put_files = {} # post上传文件 {"file_field": {"path": ["folder_name", "child_folder_name"]}}
    unique_param = [] # 需要查重的字段 ["field"]
    delete_rule = {} # 删除时传入的字段规则 {"id": [Required,],}
    post_rule = {} # 新增时传入的字段规则  {"field": [Required, Length(32), InstanceOf(str)],}
    put_rule = {} # 编辑时传入的字段规则  {"field": [Required, Length(32), InstanceOf(str)],}
    put_where_params = [] # put请求修改时的条件参数  [{"field": , "rule": "field__in"}]
    post_inlist = [] # 新增时创建model对象需要的字段 ["field1","field2"]
    put_inlist = [] # 编辑时修改model对象需要的字段 ["field1","field2"]
    file_first_path = "/usr/myfolder/" # 文件保存的初始地址
    delete_where_params = [] # delete请求删除时的条件参数 [{"field": , "rule": "field"}]


'''
常用方法
'''
import os
from django.core.paginator import Paginator

def paginationfunc(self, resList, pg, size):
    '''
    分页方法
    param resList: 需要进行分页的列表
    param pg: 页码
    param size: 每页条数
    return:
        count: 总条数
        num_pages: 总页数
        next_page: 下一页
        previous_page：上一页
        ret：当前页结果列表
    '''
    p = Paginator(resList, size)
    next_page = None
    previous_page = None
    page1 = p.page(pg)
    if page1.has_next():
        next_page = page1.next_page_number()
    if  page1.has_previous():
        previous_page = page1.previous_page_number()
    data = {"count":p.count,"num_pages":p.num_pages,"next_page":next_page,"previous_page":previous_page,
        "ret":page1.object_list}
    return data

def exam_old(modelObj, paramDict: dict, excludeDict: dict = None) -> bool:
    '''
    检查对象是否已存在 配合post新增时进行也有查重
    param modelObj: django models 对象
    param paramDict: 查重的字段以及值，{"field": field_value,}
    param excludeDict: 需要排除的条件，{"field": field_value,}
    return:
        返回bool值, 存在时返回true
    '''
    if excludeDict == None:
        old = modelObj.objects.filter(**paramDict).first()
    else:
        old = modelObj.objects.filter(**paramDict).exclude(**excludeDict).first()
    if not old:
        return False
    return True


def make_path(fpath: str):
    '''
    检查文件夹是否存在，不存在则创建
    return:
        True or err
    '''
    try:
        if not os.path.exists(fpath):
            os.makedirs(fpath)
        return True
    except Exception as e:
        return str(e)

def add_file(filePath, file):
    '''
    写入文件
    param filePath:文件写入地址
    param file: 文件对象
    return:
        True or err
    '''
    try:
        with open(filePath,'wb') as f:
            for fimg in file.chunks():
                f.write(fimg)
        return True
    except Exception as e:
        return str(e)

def file_save_addr(file, firstPath: str, pathList: list):
    '''
    对文件进行写入并返回路劲
    param file: 文件对象
    param firstPath: 初始文件夹路径
    param pathList: 文件写入的具体路径分级列表,["floder","child floder"]
    return:
        bool
        路径str
    '''
    filepath = firstPath
    filestr = ""
    for i in pathList:
        filepath = os.path.join(filepath, i)
        filestr = filestr + "/" + i
    make_path(filepath)
    lastfilePath = os.path.join(filepath, file.name)
    ok = add_file(filePath=lastfilePath, file=file)
    if ok != True:
        return ok, ""
    return True, filestr + "/" + file.name


import re
import time
from functools import wraps
from django.http import JsonResponse
from abc import ABCMeta, abstractmethod
from utils import recode

class Validator(object):
    """
    Abstract class that advanced
    validators can inherit from in order
    to set custom error messages and such.

    """

    __metaclass__ = ABCMeta

    err_message = "failed validation"
    not_message = "failed validation"

    @abstractmethod
    def __call__(self, *args, **kwargs):
        raise NotImplementedError

class NotNull(Validator):
    """
    非空验证
    # NotNull(True)
    """

    def __init__(self, base_class):
        self.base_class = base_class
        self.err_message = "参数不为空"
    def __call__(self, value):
        return True if value else False

class PhoneVerif(Validator):
    """
    手机号规范验证
    """
    def __init__(self, base_class):
        self.base_class = base_class
        self.err_message = "手机号不规范"
    def __call__(self, value):
        if value:
            if re.match(r"^1[356789]\d{9}$", value):
                return True
            return False
        return True

class DateTimeVerif(Validator):
    """
    时间格式验证
    """
    def __init__(self, base_class):
        self.base_class = base_class
        self.err_message = "时间格式不正确'YYYY-mm-dd HH:MM:SS'"
    def __call__(self, value):
        if value:
            if datetime_verify(value):
                return True
            return False
        else:
            return True

def datetime_verify(date):
    """判断是否是一个有效的日期字符串"""
    try:
        if ":" in date:
            time.strptime(date, "%Y-%m-%d %H:%M:%S")
            
        else:
            time.strptime(date, "%Y-%m-%d")
        return True
    except Exception as e:
        print(e)
        return False


class InStr(Validator):
    """
    枚举验证 字符串
    # 示例:In([1, 2, 3])
    """

    def __init__(self, collection):
        self.collection = collection
        self.err_message = "必须是 %r 其中之一" % collection

    def __call__(self, value):
        if not value:
            return True
        return (value.lower() in self.collection)


class InInt(Validator):
    """
    枚举验证 整形
    # 示例:In([1, 2, 3])
    """

    def __init__(self, collection):
        self.collection = collection
        self.err_message = "必须是 %r 其中之一" % collection

    def __call__(self, value):
        if not value:
            return True
        return (int(value) in self.collection)


class Length(Validator):
    '''
    字符长度验证
    # 示例：Length(255)
    '''
    def __init__(self, collection):
        self.collection = collection
        self.err_message = "最大长度为 %r" % collection

    def __call__(self, value):
        if value:
            return len(value) <= self.collection
        return True

class InstanceOf(Validator):
    """
    参数类型验证
    # 示例: InstanceOf(basestring)
    """

    def __init__(self, base_class):
        self.base_class = base_class
        self.err_message = "必须为 %s 类型" % base_class.__name__

    def __call__(self, value):
        if value:
            return isinstance(value, self.base_class)
        return True

class YearMonthStr(Validator):
    """
    年-月  传入验证
    # 示例: YearMonthStr(str)
    """

    def __init__(self, base_class):
        self.base_class = base_class
        self.err_message = "必须为 '年-月' 格式，如：2020-3"

    def __call__(self, value):
        s = value.split('-')
        year_len = len(s[0])
        month = int(s[1])
        return year_len == 4 and 0 < month <= 12

def Required(field, dictionary):
    if field in dictionary.keys():
        return True if dictionary[field] else False
    else:
        return False


class ValVerify(object):

    def __init__(self):
        self.error = {}

    def _validate_list_helper(self, rules, args_dict, key):
        '''
        验证:
        param(rules): 传入规则，规则的类型为字典(dict), 示例：
                      {验证字段：[规则方法1, 规则方法2]}
        param(args_dict): 需要验证的参数字典(dict), 示例：
                          {"字段1": 值,}
        param(key): 需要验证的字段
        param(err): 错误信息
        '''
        elist = []
        for vfunc in rules[key]: # 遍历这个参数所有验证方法
            if key in args_dict.keys(): # 判断这个参数有没有传入
                if vfunc != Required: # 如果验证方法不为Required
                    valid = vfunc(args_dict[key])
                    if not valid:
                        msg = getattr(vfunc, "err_message", "failed validation")
                        elist.append(msg)
        if elist != []:
            self.error[key] = elist


    def validate(self, rules, args_dict):
        '''
        验证
        param(rules): 传入规则，规则的类型为字典(dict), 示例：
                      {验证字段：[规则方法1, 规则方法2]}
        param(args_dict): 需要验证的参数字典(dict), 示例：
                          {"字段1": 值,}
        '''
        for key in rules:
            if Required in rules[key]:
                if Required(key, args_dict) != True:
                    self.error[key] = ["参数必传, 并不能为空"]
                    continue
            self._validate_list_helper(rules, args_dict, key)
        ok = True
        if self.error != {}:
            ok = False
        return ok, self.error

def validator_wrap(rules):
    """装饰器: 检测传入参数是否符合规则
    :param rules:参数的校验规则,map
    :param strip:对字段进行前后空格检测
    """

    def decorator(f):
        @wraps(f)
        def decorated_func(request, *args, **kwargs):
            try:
                if request.request.data:
                    args_dict = request.request.data
                elif request.request.GET:
                    args_dict = request.request.GET
                else:
                    args_dict = {}
                if rules:
                    result, err = ValVerify().validate(rules, args_dict)
                    if not result:
                        re_data = recode.error_func(status=22, error=err, message="验证失败")
                        return JsonResponse(re_data)
            except Exception as e:
                re_data = recode.error_func(status=33, error=str(e))
                return JsonResponse(re_data)
            return f(request, *args, **kwargs)

        return decorated_func

    return decorator

def validator_function(rules, request):
    """方法调用: 检测传入参数是否符合规则
    :param rules:参数的校验规则,map
    :param strip:对字段进行前后空格检测
    """

    try:
        if request.data:
            args_dict = request.data
        elif request.GET:
            args_dict = request.GET
        else:
            args_dict = {}
        if rules:
            result, err = ValVerify().validate(rules, args_dict)
            if not result:
                re_data = recode.error_func(status=22, error=err, message="验证失败")
                return JsonResponse(re_data)
    except Exception as e:
        re_data = recode.error_func(status=33, error=str(e))
        return JsonResponse(re_data)
    return True


from setuptools import setup, find_packages
 
setup(
    name='lminapi',
    version='0.0.2',
    keywords=("APIView", "Validator", "api"),
    description='基于django 和django rest framework整合简化drf APIView视图类常用方法, 简化传入参数已验证方法',
    long_description="整合简化drf APIView视图类常用方法, 简化传入参数已验证方法",
    license="MIT Licence",
    url='https://github.com/lxb0323/lminapi',
    author='bing0323',
    author_email='ltjlxb@163.com',
    packages= find_packages(),
    include_package_data=True,
    platforms="any",
    install_requires=['django','djangorestframework'],
)